@@ -11,12 +11,12 @@ use panic_semihosting as _;
 use stm32f4xx_hal as hal;
 
 use crate::hal::{
-    gpio::{gpioa::PA0, Edge, Input, PullDown},
+    gpio::{Edge, Input, PullDown, PA0},
     interrupt, pac,
     prelude::*,
     rcc::{Clocks, Rcc},
     spi::Spi,
-    timer::{CounterUs, Event, Timer},
+    timer::{CounterUs, Event, FTimer, Timer},
 };
 
 use core::cell::{Cell, RefCell};
@@ -101,7 +101,7 @@ fn main() -> ! {
             polarity: Polarity::IdleLow,
             phase: Phase::CaptureOnFirstTransition,
         },
-        2000.khz(),
+        2000.kHz(),
         &clocks,
     );
 
@@ -112,7 +112,7 @@ fn main() -> ! {
 
     let dc = gpioe.pe3.into_push_pull_output();
     let mut ss = gpioe.pe4.into_push_pull_output();
-    let mut delay = hal::delay::Delay::new(cp.SYST, &clocks);
+    let mut delay = Timer::syst(cp.SYST, &clocks).delay();
 
     ss.set_high();
     delay.delay_ms(100_u32);
@@ -126,9 +126,9 @@ fn main() -> ! {
     disp.flush().unwrap();
 
     // Create a 1ms periodic interrupt from TIM2
-    let mut timer = Timer::new(dp.TIM2, &clocks).counter();
+    let mut timer = FTimer::new(dp.TIM2, &clocks).counter();
     timer.start(1.secs()).unwrap();
-    timer.listen(Event::TimeOut);
+    timer.listen(Event::Update);
 
     free(|cs| {
         TIMER_TIM2.borrow(cs).replace(Some(timer));
@@ -209,10 +209,10 @@ fn main() -> ! {
 
 fn setup_clocks(rcc: Rcc) -> Clocks {
     rcc.cfgr
-        .hclk(180.mhz())
-        .sysclk(180.mhz())
-        .pclk1(45.mhz())
-        .pclk2(90.mhz())
+        .hclk(180.MHz())
+        .sysclk(180.MHz())
+        .pclk1(45.MHz())
+        .pclk2(90.MHz())
         .freeze()
 }
 
@@ -263,7 +263,7 @@ fn EXTI0() {
 fn TIM2() {
     free(|cs| {
         if let Some(ref mut tim2) = TIMER_TIM2.borrow(cs).borrow_mut().deref_mut() {
-            tim2.clear_interrupt(Event::TimeOut);
+            tim2.clear_interrupt(Event::Update);
         }
 
         let cell = ELAPSED_MS.borrow(cs);
