@@ -45,7 +45,6 @@ use cortex_m_rt::entry;
 use stm32_i2s_v12x::format::{Data16Frame16, FrameFormat};
 use stm32_i2s_v12x::{MasterClock, MasterConfig, Polarity};
 
-use stm32f4xx_hal::delay::Delay;
 use stm32f4xx_hal::i2c::I2c;
 use stm32f4xx_hal::i2s::I2s;
 use stm32f4xx_hal::nb::block;
@@ -98,9 +97,9 @@ fn main() -> ! {
 
     let rcc = dp.RCC.constrain();
     // The 86 MHz frequency can be divided to get a sample rate very close to 48 kHz.
-    let clocks = rcc.cfgr.use_hse(8.mhz()).i2s_clk(86.mhz()).freeze();
+    let clocks = rcc.cfgr.use_hse(8.MHz()).i2s_clk(86.MHz()).freeze();
 
-    let mut delay = Delay::new(cp.SYST, &clocks);
+    let mut delay = cp.SYST.delay(&clocks);
 
     let i2c = I2c::new(
         dp.I2C1,
@@ -108,7 +107,7 @@ fn main() -> ! {
             gpiob.pb6.into_alternate_open_drain(),
             gpiob.pb9.into_alternate_open_drain(),
         ),
-        100.khz(),
+        100.kHz(),
         &clocks,
     );
     // Shift the address to deal with different ways of representing I2C addresses
@@ -134,7 +133,7 @@ fn main() -> ! {
 
     let i2s = stm32_i2s_v12x::I2s::new(hal_i2s);
     let mut i2s = i2s.configure_master_transmit(MasterConfig::with_sample_rate(
-        i2s_clock.0,
+        i2s_clock.raw(),
         sample_rate,
         Data16Frame16,
         FrameFormat::PhilipsI2s,
