@@ -6,7 +6,7 @@ use cortex_m_semihosting::{hprint, hprintln};
 use panic_semihosting as _;
 
 use stm32f4xx_hal::{
-    delay, pac,
+    pac,
     prelude::*,
     sdio::{ClockFreq, Sdio},
 };
@@ -19,17 +19,17 @@ fn main() -> ! {
     let rcc = device.RCC.constrain();
     let clocks = rcc
         .cfgr
-        .use_hse(12.mhz())
+        .use_hse(12.MHz())
         .require_pll48clk()
-        .sysclk(168.mhz())
-        .hclk(168.mhz())
-        .pclk1(42.mhz())
-        .pclk2(84.mhz())
+        .sysclk(168.MHz())
+        .hclk(168.MHz())
+        .pclk1(42.MHz())
+        .pclk2(84.MHz())
         .freeze();
 
     assert!(clocks.is_pll48clk_valid());
 
-    let mut delay = delay::Delay::new(core.SYST, &clocks);
+    let mut delay = core.SYST.delay(&clocks);
 
     let gpioc = device.GPIOC.split();
     let gpiod = device.GPIOD.split();
