@@ -22,8 +22,7 @@ use panic_semihosting as _; // logs messages to the host stderr; requires a debu
 use stm32f4xx_hal as hal;
 
 use crate::hal::{
-    delay::Delay,
-    gpio::{gpioc::PC13, Edge, Input, PullUp},
+    gpio::{Edge, Input, PullUp, PC13},
     i2c::I2c,
     interrupt, pac,
     prelude::*,
@@ -72,7 +71,7 @@ fn main() -> ! {
                 gpiob.pb8.into_alternate().set_open_drain(),
                 gpiob.pb9.into_alternate().set_open_drain(),
             ),
-            400.khz(),
+            400.kHz(),
             &clocks,
         );
 
@@ -92,9 +91,9 @@ fn main() -> ! {
         disp.flush().unwrap();
 
         // Create a 1ms periodic interrupt from TIM2
-        let mut timer = Timer::new(dp.TIM2, &clocks).counter();
+        let mut timer = dp.TIM2.counter(&clocks);
         timer.start(1.secs()).unwrap();
-        timer.listen(Event::TimeOut);
+        timer.listen(Event::Update);
 
         free(|cs| {
             TIMER_TIM2.borrow(cs).replace(Some(timer));
@@ -108,7 +107,7 @@ fn main() -> ! {
             pac::NVIC::unmask(hal::pac::Interrupt::EXTI15_10);
         };
 
-        let mut delay = Delay::new(cp.SYST, &clocks);
+        let mut delay = Timer::syst(cp.SYST, &clocks).delay();
 
         loop {
             let elapsed = free(|cs| ELAPSED_MS.borrow(cs).get());
@@ -155,7 +154,7 @@ fn main() -> ! {
 fn TIM2() {
     free(|cs| {
         if let Some(ref mut tim2) = TIMER_TIM2.borrow(cs).borrow_mut().deref_mut() {
-            tim2.clear_interrupt(Event::TimeOut);
+            tim2.clear_interrupt(Event::Update);
         }
 
         let cell = ELAPSED_MS.borrow(cs);
@@ -193,10 +192,10 @@ fn EXTI15_10() {
 
 fn setup_clocks(rcc: Rcc) -> Clocks {
     rcc.cfgr
-        .hclk(48.mhz())
-        .sysclk(48.mhz())
-        .pclk1(24.mhz())
-        .pclk2(24.mhz())
+        .hclk(48.MHz())
+        .sysclk(48.MHz())
+        .pclk1(24.MHz())
+        .pclk2(24.MHz())
         .freeze()
 }
 
