@@ -15,8 +15,7 @@ use panic_halt as _;
 use cortex_m_rt::entry;
 use cortex_m_semihosting::hprintln;
 
-use hal::timer;
-use hal::timer::Timer;
+use hal::timer::Error;
 use stm32f4xx_hal as hal;
 
 use crate::hal::{pac, prelude::*};
@@ -25,10 +24,10 @@ use crate::hal::{pac, prelude::*};
 fn main() -> ! {
     let dp = pac::Peripherals::take().unwrap();
     let rcc = dp.RCC.constrain();
-    let clocks = rcc.cfgr.sysclk(24.mhz()).freeze();
+    let clocks = rcc.cfgr.sysclk(24.MHz()).freeze();
 
     // Create a timer based on SysTick
-    let mut timer = Timer::new(dp.TIM1, &clocks).counter_ms();
+    let mut timer = dp.TIM1.counter_ms(&clocks);
     timer.start(1.secs()).unwrap();
 
     hprintln!("hello!").unwrap();
@@ -36,7 +35,7 @@ fn main() -> ! {
     nb::block!(timer.wait()).unwrap();
     hprintln!("timer expired 1").unwrap();
 
-    // the function syst() creates a periodic timer, so it is automatically
+    // the function counter_ms() creates a periodic timer, so it is automatically
     // restarted
     nb::block!(timer.wait()).unwrap();
     hprintln!("timer expired 2").unwrap();
@@ -45,12 +44,13 @@ fn main() -> ! {
     timer.cancel().unwrap();
 
     // start it again
-    timer.delay(1.secs()).unwrap();
+    timer.start(1.secs()).unwrap();
+    nb::block!(timer.wait()).unwrap();
     hprintln!("timer expired 3").unwrap();
 
     timer.cancel().unwrap();
     let cancel_outcome = timer.cancel();
-    assert_eq!(cancel_outcome, Err(timer::Error::Disabled));
+    assert_eq!(cancel_outcome, Err(Error::Disabled));
     hprintln!("ehy, you cannot cancel a timer two times!").unwrap();
     // this time the timer was not restarted, therefore this function should
     // wait forever
