@@ -16,6 +16,9 @@ pub(crate) mod sealed {
 }
 use sealed::{Bits, Sealed};
 
+/// Marker trait for structs which can be safely accessed with shared reference
+pub trait SafePeripheralRead {}
+
 /// Trait for DMA stream interrupt handling.
 pub trait StreamISR: Sealed {
     /// Clear all interrupts for the DMA stream.
@@ -931,6 +934,10 @@ dma_map!(
     (Stream2<DMA2>, 1, pac::ADC2, PeripheralToMemory),  //ADC2
     (Stream3<DMA2>, 1, pac::ADC2, PeripheralToMemory),  //ADC2
     (Stream7<DMA2>, 1, pac::DCMI, PeripheralToMemory),  //DCMI
+    (Stream2<DMA2>, 1, Adc<pac::ADC3>, PeripheralToMemory), //ADC2
+    (Stream3<DMA2>, 1, Adc<pac::ADC3>, PeripheralToMemory), //ADC2
+    (Stream0<DMA2>, 2, Adc<pac::ADC3>, PeripheralToMemory), //ADC3
+    (Stream1<DMA2>, 2, Adc<pac::ADC3>, PeripheralToMemory), //ADC3
 );
 #[cfg(any(
     feature = "stm32f417",
