@@ -70,25 +70,15 @@ pub use erased::{EPin, ErasedPin};
 mod dynamic;
 pub use dynamic::{Dynamic, DynamicPin};
 mod hal_02;
+mod hal_1;
 
-#[derive(Debug, PartialEq, Eq, Clone, Copy)]
-pub enum PinState {
-    /// Low pin state
-    Low,
-    /// High pin state
-    High,
-}
+pub use embedded_hal::digital::v2::PinState;
 
-impl From<bool> for PinState {
-    fn from(value: bool) -> Self {
-        match value {
-            false => PinState::Low,
-            true => PinState::High,
-        }
-    }
-}
+use core::fmt;
 
 /// A filler pin type
+#[derive(Debug)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct NoPin;
 
 /// Extension trait to split a GPIO peripheral in independent pins and registers
@@ -142,6 +132,7 @@ pub struct Analog;
 pub type Debugger = Alternate<PushPull, 0>;
 
 /// GPIO Pin speed selection
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Speed {
     Low = 0,
@@ -150,6 +141,7 @@ pub enum Speed {
     VeryHigh = 3,
 }
 
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Edge {
     Rising,
@@ -279,6 +271,24 @@ impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
     }
 }
 
+impl<MODE, const P: char, const N: u8> fmt::Debug for Pin<MODE, P, N> {
+    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_fmt(format_args!(
+            "P{}{}<{}>",
+            P,
+            N,
+            crate::stripped_type_name::<MODE>()
+        ))
+    }
+}
+
+#[cfg(feature = "defmt")]
+impl<MODE, const P: char, const N: u8> defmt::Format for Pin<MODE, P, N> {
+    fn format(&self, f: defmt::Formatter) {
+        defmt::write!(f, "P{}{}<{}>", P, N, crate::stripped_type_name::<MODE>());
+    }
+}
+
 impl<MODE, const P: char, const N: u8> PinExt for Pin<MODE, P, N> {
     type Mode = MODE;
 
@@ -561,6 +571,8 @@ macro_rules! gpio {
             )+
 
         }
+
+        pub use $gpiox::{ $($PXi,)+ };
     }
 }
 
