@@ -8,7 +8,7 @@ impl<const L: u8, const R: u8> Assert<L, R> {
 }
 
 impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Input<MODE>, P, N>>
-    for Pin<Alternate<A>, P, N>
+    for Pin<Alternate<PushPull, A>, P, N>
 {
     #[inline(always)]
     fn from(f: Pin<Input<MODE>, P, N>) -> Self {
@@ -17,7 +17,7 @@ impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Input<MODE>, P, N>>
 }
 
 impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Output<MODE>, P, N>>
-    for Pin<Alternate<A>, P, N>
+    for Pin<Alternate<PushPull, A>, P, N>
 {
     #[inline(always)]
     fn from(f: Pin<Output<MODE>, P, N>) -> Self {
@@ -25,24 +25,26 @@ impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Output<MODE>, P, N>
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Analog, P, N>> for Pin<Alternate<A>, P, N> {
+impl<const P: char, const N: u8, const A: u8> From<Pin<Analog, P, N>>
+    for Pin<Alternate<PushPull, A>, P, N>
+{
     #[inline(always)]
     fn from(f: Pin<Analog, P, N>) -> Self {
         f.into_alternate::<A>()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8, const B: u8> From<Pin<AlternateOD<B>, P, N>>
-    for Pin<Alternate<A>, P, N>
+impl<const P: char, const N: u8, const A: u8, const B: u8> From<Pin<Alternate<OpenDrain, B>, P, N>>
+    for Pin<Alternate<PushPull, A>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<B>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, B>, P, N>) -> Self {
         f.into_alternate::<A>()
     }
 }
 
 impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Input<MODE>, P, N>>
-    for Pin<AlternateOD<A>, P, N>
+    for Pin<Alternate<OpenDrain, A>, P, N>
 {
     #[inline(always)]
     fn from(f: Pin<Input<MODE>, P, N>) -> Self {
@@ -51,7 +53,7 @@ impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Input<MODE>, P, N>>
 }
 
 impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Output<MODE>, P, N>>
-    for Pin<AlternateOD<A>, P, N>
+    for Pin<Alternate<OpenDrain, A>, P, N>
 {
     #[inline(always)]
     fn from(f: Pin<Output<MODE>, P, N>) -> Self {
@@ -60,7 +62,7 @@ impl<MODE, const P: char, const N: u8, const A: u8> From<Pin<Output<MODE>, P, N>
 }
 
 impl<const P: char, const N: u8, const A: u8> From<Pin<Analog, P, N>>
-    for Pin<AlternateOD<A>, P, N>
+    for Pin<Alternate<OpenDrain, A>, P, N>
 {
     #[inline(always)]
     fn from(f: Pin<Analog, P, N>) -> Self {
@@ -68,11 +70,11 @@ impl<const P: char, const N: u8, const A: u8> From<Pin<Analog, P, N>>
     }
 }
 
-impl<const P: char, const N: u8, const A: u8, const B: u8> From<Pin<Alternate<B>, P, N>>
-    for Pin<AlternateOD<A>, P, N>
+impl<const P: char, const N: u8, const A: u8, const B: u8> From<Pin<Alternate<PushPull, B>, P, N>>
+    for Pin<Alternate<OpenDrain, A>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<B>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, B>, P, N>) -> Self {
         f.into_alternate_open_drain::<A>()
     }
 }
@@ -107,20 +109,20 @@ impl<const P: char, const N: u8> From<Pin<Analog, P, N>> for Pin<Input<PullDown>
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
     for Pin<Input<PullDown>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_pull_down_input()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Input<PullDown>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_pull_down_input()
     }
 }
@@ -153,20 +155,20 @@ impl<const P: char, const N: u8> From<Pin<Analog, P, N>> for Pin<Input<PullUp>,
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
     for Pin<Input<PullUp>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_pull_up_input()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Input<PullUp>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_pull_up_input()
     }
 }
@@ -201,20 +203,20 @@ impl<const P: char, const N: u8> From<Pin<Analog, P, N>> for Pin<Input<Floating>
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
     for Pin<Input<Floating>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_floating_input()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Input<Floating>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_floating_input()
     }
 }
@@ -244,20 +246,20 @@ impl<const P: char, const N: u8> From<Pin<Analog, P, N>> for Pin<Output<OpenDrai
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
     for Pin<Output<OpenDrain>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_open_drain_output()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Output<OpenDrain>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_open_drain_output()
     }
 }
@@ -287,20 +289,20 @@ impl<const P: char, const N: u8> From<Pin<Analog, P, N>> for Pin<Output<PushPull
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
     for Pin<Output<PushPull>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_push_pull_output()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Output<PushPull>, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_push_pull_output()
     }
 }
@@ -319,24 +321,26 @@ impl<MODE, const P: char, const N: u8> From<Pin<Output<MODE>, P, N>> for Pin<Ana
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<A>, P, N>> for Pin<Analog, P, N> {
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<PushPull, A>, P, N>>
+    for Pin<Analog, P, N>
+{
     #[inline(always)]
-    fn from(f: Pin<Alternate<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<PushPull, A>, P, N>) -> Self {
         f.into_analog()
     }
 }
 
-impl<const P: char, const N: u8, const A: u8> From<Pin<AlternateOD<A>, P, N>>
+impl<const P: char, const N: u8, const A: u8> From<Pin<Alternate<OpenDrain, A>, P, N>>
     for Pin<Analog, P, N>
 {
     #[inline(always)]
-    fn from(f: Pin<AlternateOD<A>, P, N>) -> Self {
+    fn from(f: Pin<Alternate<OpenDrain, A>, P, N>) -> Self {
         f.into_analog()
     }
 }
 
 impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
-    fn set_alternate<const A: u8>(&mut self) {
+    pub(super) fn set_alternate<const A: u8>(&mut self) {
         #[allow(path_statements, clippy::no_effect)]
         {
             Assert::<A, 16>::LESS;
@@ -360,208 +364,17 @@ impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
         }
     }
     /// Configures the pin to operate alternate mode
-    pub fn into_alternate<const A: u8>(mut self) -> Pin<Alternate<A>, P, N> {
+    pub fn into_alternate<const A: u8>(mut self) -> Pin<Alternate<PushPull, A>, P, N> {
         self.set_alternate::<A>();
         Pin::new()
     }
 
     /// Configures the pin to operate in alternate open drain mode
     #[allow(path_statements)]
-    pub fn into_alternate_open_drain<const A: u8>(self) -> Pin<AlternateOD<A>, P, N> {
+    pub fn into_alternate_open_drain<const A: u8>(self) -> Pin<Alternate<OpenDrain, A>, P, N> {
         self.into_alternate::<A>().set_open_drain()
     }
 
-    /// Configures the pin to operate in AF0 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af0(self) -> Pin<Alternate<0>, P, N> {
-        self.into_alternate::<0>()
-    }
-
-    /// Configures the pin to operate in AF1 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af1(self) -> Pin<Alternate<1>, P, N> {
-        self.into_alternate::<1>()
-    }
-
-    /// Configures the pin to operate in AF2 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af2(self) -> Pin<Alternate<2>, P, N> {
-        self.into_alternate::<2>()
-    }
-
-    /// Configures the pin to operate in AF3 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af3(self) -> Pin<Alternate<3>, P, N> {
-        self.into_alternate::<3>()
-    }
-
-    /// Configures the pin to operate in AF4 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af4(self) -> Pin<Alternate<4>, P, N> {
-        self.into_alternate::<4>()
-    }
-
-    /// Configures the pin to operate in AF5 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af5(self) -> Pin<Alternate<5>, P, N> {
-        self.into_alternate::<5>()
-    }
-
-    /// Configures the pin to operate in AF6 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af6(self) -> Pin<Alternate<6>, P, N> {
-        self.into_alternate::<6>()
-    }
-
-    /// Configures the pin to operate in AF7 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af7(self) -> Pin<Alternate<7>, P, N> {
-        self.into_alternate::<7>()
-    }
-
-    /// Configures the pin to operate in AF8 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af8(self) -> Pin<Alternate<8>, P, N> {
-        self.into_alternate::<8>()
-    }
-
-    /// Configures the pin to operate in AF9 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af9(self) -> Pin<Alternate<9>, P, N> {
-        self.into_alternate::<9>()
-    }
-
-    /// Configures the pin to operate in AF10 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af10(self) -> Pin<Alternate<10>, P, N> {
-        self.into_alternate::<10>()
-    }
-
-    /// Configures the pin to operate in AF11 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af11(self) -> Pin<Alternate<11>, P, N> {
-        self.into_alternate::<11>()
-    }
-
-    /// Configures the pin to operate in AF12 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af12(self) -> Pin<Alternate<12>, P, N> {
-        self.into_alternate::<12>()
-    }
-
-    /// Configures the pin to operate in AF13 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af13(self) -> Pin<Alternate<13>, P, N> {
-        self.into_alternate::<13>()
-    }
-
-    /// Configures the pin to operate in AF14 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af14(self) -> Pin<Alternate<14>, P, N> {
-        self.into_alternate::<14>()
-    }
-
-    /// Configures the pin to operate in AF15 mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af15(self) -> Pin<Alternate<15>, P, N> {
-        self.into_alternate::<15>()
-    }
-
-    /// Configures the pin to operate in AF0 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af0_open_drain(self) -> Pin<AlternateOD<0>, P, N> {
-        self.into_alternate_open_drain::<0>()
-    }
-
-    /// Configures the pin to operate in AF1 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af1_open_drain(self) -> Pin<AlternateOD<1>, P, N> {
-        self.into_alternate_open_drain::<1>()
-    }
-
-    /// Configures the pin to operate in AF2 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af2_open_drain(self) -> Pin<AlternateOD<2>, P, N> {
-        self.into_alternate_open_drain::<2>()
-    }
-
-    /// Configures the pin to operate in AF3 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af3_open_drain(self) -> Pin<AlternateOD<3>, P, N> {
-        self.into_alternate_open_drain::<3>()
-    }
-
-    /// Configures the pin to operate in AF4 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af4_open_drain(self) -> Pin<AlternateOD<4>, P, N> {
-        self.into_alternate_open_drain::<4>()
-    }
-
-    /// Configures the pin to operate in AF5 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af5_open_drain(self) -> Pin<AlternateOD<5>, P, N> {
-        self.into_alternate_open_drain::<5>()
-    }
-
-    /// Configures the pin to operate in AF6 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af6_open_drain(self) -> Pin<AlternateOD<6>, P, N> {
-        self.into_alternate_open_drain::<6>()
-    }
-
-    /// Configures the pin to operate in AF7 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af7_open_drain(self) -> Pin<AlternateOD<7>, P, N> {
-        self.into_alternate_open_drain::<7>()
-    }
-
-    /// Configures the pin to operate in AF8 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af8_open_drain(self) -> Pin<AlternateOD<8>, P, N> {
-        self.into_alternate_open_drain::<8>()
-    }
-
-    /// Configures the pin to operate in AF9 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af9_open_drain(self) -> Pin<AlternateOD<9>, P, N> {
-        self.into_alternate_open_drain::<9>()
-    }
-
-    /// Configures the pin to operate in AF10 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af10_open_drain(self) -> Pin<AlternateOD<10>, P, N> {
-        self.into_alternate_open_drain::<10>()
-    }
-
-    /// Configures the pin to operate in AF11 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af11_open_drain(self) -> Pin<AlternateOD<11>, P, N> {
-        self.into_alternate_open_drain::<11>()
-    }
-
-    /// Configures the pin to operate in AF12 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af12_open_drain(self) -> Pin<AlternateOD<12>, P, N> {
-        self.into_alternate_open_drain::<12>()
-    }
-
-    /// Configures the pin to operate in AF13 open drain modev
-    pub fn into_alternate_af13_open_drain(self) -> Pin<AlternateOD<13>, P, N> {
-        self.into_alternate_open_drain::<13>()
-    }
-
-    /// Configures the pin to operate in AF14 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af14_open_drain(self) -> Pin<AlternateOD<14>, P, N> {
-        self.into_alternate_open_drain::<14>()
-    }
-
-    /// Configures the pin to operate in AF15 open drain mode
-    #[deprecated(since = "0.10.0")]
-    pub fn into_alternate_af15_open_drain(self) -> Pin<AlternateOD<15>, P, N> {
-        self.into_alternate_open_drain::<15>()
-    }
-
     /// Configures the pin to operate as a floating input pin
     pub fn into_floating_input(mut self) -> Pin<Input<Floating>, P, N> {
         self.mode::<Input<Floating>>();
@@ -623,12 +436,20 @@ impl<MODE, const P: char, const N: u8> Pin<MODE, P, N> {
         Pin::new()
     }
 
+    /// Configures the pin as a pin that can change between input
+    /// and output without changing the type. It starts out
+    /// as a floating input
+    pub fn into_dynamic(self) -> DynamicPin<P, N> {
+        self.into_floating_input();
+        DynamicPin::new(Dynamic::InputFloating)
+    }
+
     /// Puts `self` into mode `M`.
     ///
     /// This violates the type state constraints from `MODE`, so callers must
     /// ensure they use this properly.
     #[inline(always)]
-    fn mode<M: PinMode>(&mut self) {
+    pub(super) fn mode<M: PinMode>(&mut self) {
         let offset = 2 * N;
         unsafe {
             (*Gpio::<P>::ptr())
@@ -830,56 +651,3 @@ impl PinMode for Output<PushPull> {
     const MODER: u32 = 0b01;
     const OTYPER: Option<u32> = Some(0b0);
 }
-
-pub struct Const<const A: u8>;
-
-pub trait SetAlternate<const A: u8> {
-    fn set_alt_mode(&mut self);
-    fn restore_mode(&mut self);
-}
-impl SetAlternate<0> for NoPin {
-    fn set_alt_mode(&mut self) {}
-    fn restore_mode(&mut self) {}
-}
-impl<MODE: PinMode, const P: char, const N: u8, const A: u8> SetAlternate<A> for Pin<MODE, P, N> {
-    fn set_alt_mode(&mut self) {
-        self.set_alternate::<A>();
-    }
-
-    fn restore_mode(&mut self) {
-        self.mode::<MODE>();
-    }
-}
-
-impl<const P: char, const N: u8, const A: u8> SetAlternate<A> for Pin<Alternate<A>, P, N> {
-    fn set_alt_mode(&mut self) {}
-    fn restore_mode(&mut self) {}
-}
-
-pub trait SetAlternateOD<const A: u8> {
-    fn set_alt_mode(&mut self);
-    fn restore_mode(&mut self);
-}
-impl SetAlternateOD<0> for NoPin {
-    fn set_alt_mode(&mut self) {}
-    fn restore_mode(&mut self) {}
-}
-impl<MODE: PinMode, const P: char, const N: u8, const A: u8> SetAlternateOD<A> for Pin<MODE, P, N> {
-    fn set_alt_mode(&mut self) {
-        self.set_alternate::<A>();
-        unsafe {
-            (*Gpio::<P>::ptr())
-                .otyper
-                .modify(|r, w| w.bits(r.bits() | (1 << N)))
-        };
-    }
-
-    fn restore_mode(&mut self) {
-        self.mode::<MODE>();
-    }
-}
-
-impl<const P: char, const N: u8, const A: u8> SetAlternateOD<A> for Pin<AlternateOD<A>, P, N> {
-    fn set_alt_mode(&mut self) {}
-    fn restore_mode(&mut self) {}
-}
