@@ -23,26 +23,11 @@ use crate::rcc;
 use nb::block;
 
 mod hal_02;
+mod hal_1;
 
 use crate::gpio::{Const, PinA, PushPull, SetAlternate};
 
-use crate::pac::{RCC, USART1, USART2, USART6};
-
-#[cfg(feature = "usart3")]
-use crate::pac::USART3;
-
-#[cfg(feature = "uart10")]
-use crate::pac::UART10;
-#[cfg(feature = "uart4")]
-use crate::pac::UART4;
-#[cfg(feature = "uart5")]
-use crate::pac::UART5;
-#[cfg(feature = "uart7")]
-use crate::pac::UART7;
-#[cfg(feature = "uart8")]
-use crate::pac::UART8;
-#[cfg(feature = "uart9")]
-use crate::pac::UART9;
+use crate::pac::{self, RCC};
 
 use crate::gpio::NoPin;
 use crate::rcc::Clocks;
@@ -50,18 +35,7 @@ use crate::rcc::Clocks;
 use crate::dma::traits::PeriAddress;
 
 /// Serial error
-#[non_exhaustive]
-#[derive(Debug, Eq, PartialEq, Copy, Clone)]
-pub enum Error {
-    /// Framing error
-    Framing,
-    /// Noise error
-    Noise,
-    /// RX buffer overrun
-    Overrun,
-    /// Parity check error
-    Parity,
-}
+pub use embedded_hal_one::serial::ErrorKind as Error;
 
 /// Interrupt event
 pub enum Event {
@@ -77,17 +51,34 @@ pub mod config {
     use crate::time::Bps;
     use crate::time::U32Ext;
 
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
+    #[derive(Debug, Clone, Copy, PartialEq)]
     pub enum WordLength {
         DataBits8,
         DataBits9,
     }
 
+    /// Parity generation and checking. If odd or even parity is selected, the
+    /// underlying USART will be configured to send/receive the parity bit in
+    /// addtion to the data bits.
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
+    #[derive(Debug, Clone, Copy, PartialEq)]
     pub enum Parity {
+        /// No parity bit will be added/checked.
         ParityNone,
+        /// The MSB transmitted/received will be generated/checked to have a
+        /// even number of bits set.
         ParityEven,
+        /// The MSB transmitted/received will be generated/checked to have a
+        /// odd number of bits set.
         ParityOdd,
     }
 
+    /// Stop Bit configuration parameter for serial.
+    ///
+    /// Wrapper around [`STOP_A`]
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
+    #[derive(Clone, Copy, Debug, PartialEq)]
     pub enum StopBits {
         #[doc = "1 stop bit"]
         STOP1,
@@ -99,6 +90,8 @@ pub mod config {
         STOP1P5,
     }
 
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
+    #[derive(Debug, Clone, Copy, PartialEq)]
     pub enum DmaConfig {
         None,
         Tx,
@@ -106,6 +99,9 @@ pub mod config {
         TxRx,
     }
 
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
+    #[derive(Debug, Clone, Copy, PartialEq)]
+    #[non_exhaustive]
     pub struct Config {
         pub baudrate: Bps,
         pub wordlength: WordLength,
@@ -336,6 +332,64 @@ impl<USART, PINS, WORD> AsMut<Rx<USART, WORD>> for Serial<USART, PINS, WORD> {
     }
 }
 
+pub trait SerialExt: Sized + Instance {
+    fn serial<PINS: Pins<Self>, WORD>(
+        self,
+        pins: PINS,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Serial<Self, PINS, WORD>, config::InvalidConfig>;
+    fn tx<TX, WORD>(
+        self,
+        tx_pin: TX,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Tx<Self, WORD>, config::InvalidConfig>
+    where
+        (TX, NoPin): Pins<Self>;
+    fn rx<RX, WORD>(
+        self,
+        rx_pin: RX,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Rx<Self, WORD>, config::InvalidConfig>
+    where
+        (NoPin, RX): Pins<Self>;
+}
+
+impl<USART: Instance> SerialExt for USART {
+    fn serial<PINS: Pins<Self>, WORD>(
+        self,
+        pins: PINS,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Serial<Self, PINS, WORD>, config::InvalidConfig> {
+        Serial::new(self, pins, config, clocks)
+    }
+    fn tx<TX, WORD>(
+        self,
+        tx_pin: TX,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Tx<Self, WORD>, config::InvalidConfig>
+    where
+        (TX, NoPin): Pins<Self>,
+    {
+        Serial::tx(self, tx_pin, config, clocks)
+    }
+    fn rx<RX, WORD>(
+        self,
+        rx_pin: RX,
+        config: impl Into<config::Config>,
+        clocks: &Clocks,
+    ) -> Result<Rx<Self, WORD>, config::InvalidConfig>
+    where
+        (NoPin, RX): Pins<Self>,
+    {
+        Serial::rx(self, rx_pin, config, clocks)
+    }
+}
+
 impl<USART, PINS, WORD> Serial<USART, PINS, WORD>
 where
     PINS: Pins<USART>,
@@ -364,7 +418,7 @@ where
             USART::reset(rcc);
         }
 
-        let pclk_freq = USART::clock(clocks).0;
+        let pclk_freq = USART::clock(clocks).raw();
         let baud = config.baudrate.0;
 
         // The frequency to calculate USARTDIV is this:
@@ -474,9 +528,9 @@ where
     }
 }
 
-impl<USART, TX, WORD, const TXA: u8> Serial<USART, (TX, NoPin), WORD>
+impl<USART, TX, WORD> Serial<USART, (TX, NoPin), WORD>
 where
-    TX: PinA<TxPin, USART, A = Const<TXA>> + SetAlternate<PushPull, TXA>,
+    (TX, NoPin): Pins<USART>,
     USART: Instance,
 {
     pub fn tx(
@@ -489,9 +543,9 @@ where
     }
 }
 
-impl<USART, RX, WORD, const RXA: u8> Serial<USART, (NoPin, RX), WORD>
+impl<USART, RX, WORD> Serial<USART, (NoPin, RX), WORD>
 where
-    RX: PinA<RxPin, USART, A = Const<RXA>> + SetAlternate<PushPull, RXA>,
+    (NoPin, RX): Pins<USART>,
     USART: Instance,
 {
     pub fn rx(
@@ -664,10 +718,14 @@ pub trait Instance: crate::Sealed + rcc::Enable + rcc::Reset + rcc::BusClock {
 }
 
 macro_rules! halUsart {
-    ($USARTX:ty) => {
-        impl Instance for $USARTX {
+    ($USART:ty, $Serial:ident, $Tx:ident, $Rx:ident) => {
+        pub type $Serial<PINS> = Serial<$USART, PINS, u8>;
+        pub type $Tx = Tx<$USART, u8>;
+        pub type $Rx = Rx<$USART, u8>;
+
+        impl Instance for $USART {
             fn ptr() -> *const uart_base::RegisterBlock {
-                <$USARTX>::ptr() as *const _
+                <$USART>::ptr() as *const _
             }
 
             fn set_stopbits(&self, bits: config::StopBits) {
@@ -698,10 +756,14 @@ macro_rules! halUsart {
 ))]
 #[cfg(not(any(feature = "stm32f413", feature = "stm32f423",)))]
 macro_rules! halUart {
-    ($USARTX:ty) => {
-        impl Instance for $USARTX {
+    ($USART:ty, $Serial:ident, $Tx:ident, $Rx:ident) => {
+        pub type $Serial<PINS> = Serial<$USART, PINS, u8>;
+        pub type $Tx = Tx<$USART, u8>;
+        pub type $Rx = Rx<$USART, u8>;
+
+        impl Instance for $USART {
             fn ptr() -> *const uart_base::RegisterBlock {
-                <$USARTX>::ptr() as *const _
+                <$USART>::ptr() as *const _
             }
 
             fn set_stopbits(&self, bits: config::StopBits) {
@@ -721,53 +783,144 @@ macro_rules! halUart {
     };
 }
 
-halUsart! { USART1 }
-halUsart! { USART2 }
-halUsart! { USART6 }
+halUsart! { pac::USART1, Serial1, Rx1, Tx1 }
+halUsart! { pac::USART2, Serial2, Rx2, Tx2 }
+halUsart! { pac::USART6, Serial6, Rx6, Tx6 }
 
 #[cfg(feature = "usart3")]
-halUsart! { USART3 }
+halUsart! { pac::USART3, Serial3, Rx3, Tx3 }
 
 #[cfg(feature = "uart4")]
 #[cfg(not(any(feature = "stm32f413", feature = "stm32f423")))]
-halUart! { UART4 }
+halUart! { pac::UART4, Serial4, Rx4, Tx4 }
 #[cfg(feature = "uart5")]
 #[cfg(not(any(feature = "stm32f413", feature = "stm32f423")))]
-halUart! { UART5 }
+halUart! { pac::UART5, Serial5, Rx5, Tx5 }
 
 #[cfg(feature = "uart4")]
 #[cfg(any(feature = "stm32f413", feature = "stm32f423"))]
-halUsart! { UART4 }
+halUsart! { pac::UART4, Serial4, Rx4, Tx4 }
 #[cfg(feature = "uart5")]
 #[cfg(any(feature = "stm32f413", feature = "stm32f423"))]
-halUsart! { UART5 }
+halUsart! { pac::UART5, Serial5, Rx5, Tx5 }
 
 #[cfg(feature = "uart7")]
-halUsart! { UART7 }
+halUsart! { pac::UART7, Serial7, Rx7, Tx7 }
 #[cfg(feature = "uart8")]
-halUsart! { UART8 }
+halUsart! { pac::UART8, Serial8, Rx8, Tx8 }
 #[cfg(feature = "uart9")]
-halUsart! { UART9 }
+halUsart! { pac::UART9, Serial9, Rx9, Tx9 }
 #[cfg(feature = "uart10")]
-halUsart! { UART10 }
+halUsart! { pac::UART10, Serial10, Rx10, Tx10 }
 
-impl<USART, PINS> fmt::Write for Serial<USART, PINS>
-where
-    Tx<USART>: embedded_hal::serial::Write<u8>,
-{
+impl<USART: Instance, PINS> fmt::Write for Serial<USART, PINS> {
     fn write_str(&mut self, s: &str) -> fmt::Result {
         self.tx.write_str(s)
     }
 }
 
-impl<USART> fmt::Write for Tx<USART>
-where
-    Tx<USART>: embedded_hal::serial::Write<u8>,
-{
+impl<USART: Instance> fmt::Write for Tx<USART> {
     fn write_str(&mut self, s: &str) -> fmt::Result {
-        use embedded_hal::serial::Write;
         s.bytes()
             .try_for_each(|c| block!(self.write(c)))
             .map_err(|_| fmt::Error)
     }
 }
+
+impl<USART: Instance> Rx<USART, u8> {
+    fn read(&mut self) -> nb::Result<u8, Error> {
+        // Delegate to the Read<u16> implementation, then truncate to 8 bits
+        Rx::<USART, u16>::new().read().map(|word16| word16 as u8)
+    }
+}
+
+impl<USART: Instance> Rx<USART, u16> {
+    fn read(&mut self) -> nb::Result<u16, Error> {
+        // NOTE(unsafe) atomic read with no side effects
+        let sr = unsafe { (*USART::ptr()).sr.read() };
+
+        // Any error requires the dr to be read to clear
+        if sr.pe().bit_is_set()
+            || sr.fe().bit_is_set()
+            || sr.nf().bit_is_set()
+            || sr.ore().bit_is_set()
+        {
+            unsafe { (*USART::ptr()).dr.read() };
+        }
+
+        Err(if sr.pe().bit_is_set() {
+            Error::Parity.into()
+        } else if sr.fe().bit_is_set() {
+            Error::FrameFormat.into()
+        } else if sr.nf().bit_is_set() {
+            Error::Noise.into()
+        } else if sr.ore().bit_is_set() {
+            Error::Overrun.into()
+        } else if sr.rxne().bit_is_set() {
+            // NOTE(unsafe) atomic read from stateless register
+            return Ok(unsafe { &*USART::ptr() }.dr.read().dr().bits());
+        } else {
+            nb::Error::WouldBlock
+        })
+    }
+}
+
+impl<USART: Instance> Tx<USART, u8> {
+    fn write(&mut self, word: u8) -> nb::Result<(), Error> {
+        // Delegate to u16 version
+        Tx::<USART, u16>::new().write(u16::from(word))
+    }
+
+    fn flush(&mut self) -> nb::Result<(), Error> {
+        // Delegate to u16 version
+        Tx::<USART, u16>::new().flush()
+    }
+
+    fn bwrite_all(&mut self, bytes: &[u8]) -> Result<(), Error> {
+        for &b in bytes {
+            nb::block!(self.write(b))?;
+        }
+        Ok(())
+    }
+
+    fn bflush(&mut self) -> Result<(), Error> {
+        nb::block!(self.flush())
+    }
+}
+
+impl<USART: Instance> Tx<USART, u16> {
+    fn write(&mut self, word: u16) -> nb::Result<(), Error> {
+        // NOTE(unsafe) atomic read with no side effects
+        let sr = unsafe { (*USART::ptr()).sr.read() };
+
+        if sr.txe().bit_is_set() {
+            // NOTE(unsafe) atomic write to stateless register
+            unsafe { &*USART::ptr() }.dr.write(|w| w.dr().bits(word));
+            Ok(())
+        } else {
+            Err(nb::Error::WouldBlock)
+        }
+    }
+
+    fn flush(&mut self) -> nb::Result<(), Error> {
+        // NOTE(unsafe) atomic read with no side effects
+        let sr = unsafe { (*USART::ptr()).sr.read() };
+
+        if sr.tc().bit_is_set() {
+            Ok(())
+        } else {
+            Err(nb::Error::WouldBlock)
+        }
+    }
+
+    fn bwrite_all(&mut self, buffer: &[u16]) -> Result<(), Error> {
+        for &b in buffer {
+            nb::block!(self.write(b))?;
+        }
+        Ok(())
+    }
+
+    fn bflush(&mut self) -> Result<(), Error> {
+        nb::block!(self.flush())
+    }
+}
