@@ -4,6 +4,7 @@ use core::ptr;
 
 use crate::dma::traits::PeriAddress;
 use crate::gpio::{Const, NoPin, PinA, PushPull, SetAlternate};
+use crate::pac;
 
 /// Clock polarity
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -33,28 +34,18 @@ pub struct Mode {
 }
 
 mod hal_02;
+mod hal_1;
 
-use crate::pac::{spi1, RCC, SPI1, SPI2};
+use crate::pac::{spi1, RCC};
 use crate::rcc;
 
-#[cfg(feature = "spi3")]
-use crate::pac::SPI3;
-
-#[cfg(feature = "spi4")]
-use crate::pac::SPI4;
-
-#[cfg(feature = "spi5")]
-use crate::pac::SPI5;
-
-#[cfg(feature = "spi6")]
-use crate::pac::SPI6;
-
 use crate::rcc::Clocks;
-use crate::time::Hertz;
+use fugit::HertzU32 as Hertz;
 
 /// SPI error
-#[non_exhaustive]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[derive(Debug, Eq, PartialEq, Copy, Clone)]
+#[non_exhaustive]
 pub enum Error {
     /// Overrun occurred
     Overrun,
@@ -105,6 +96,8 @@ pub type NoMiso = NoPin;
 pub type NoMosi = NoPin;
 
 /// Interrupt events
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
+#[derive(Debug, Eq, PartialEq, Copy, Clone)]
 pub enum Event {
     /// New data has been received
     Rxne,
@@ -136,7 +129,9 @@ pub trait Instance:
 
 // Implemented by all SPI instances
 macro_rules! spi {
-    ($SPI:ident: ($spi:ident)) => {
+    ($SPI:ty: $Spi:ident) => {
+        pub type $Spi<PINS> = Spi<$SPI, PINS, TransferModeNormal>;
+
         impl Instance for $SPI {
             fn ptr() -> *const spi1::RegisterBlock {
                 <$SPI>::ptr() as *const _
@@ -145,20 +140,58 @@ macro_rules! spi {
     };
 }
 
-spi! { SPI1: (spi1) }
-spi! { SPI2: (spi2) }
+spi! { pac::SPI1: Spi1 }
+spi! { pac::SPI2: Spi2 }
 
 #[cfg(feature = "spi3")]
-spi! { SPI3: (spi3) }
+spi! { pac::SPI3: Spi3 }
 
 #[cfg(feature = "spi4")]
-spi! { SPI4: (spi4) }
+spi! { pac::SPI4: Spi4 }
 
 #[cfg(feature = "spi5")]
-spi! { SPI5: (spi5) }
+spi! { pac::SPI5: Spi5 }
 
 #[cfg(feature = "spi6")]
-spi! { SPI6: (spi6) }
+spi! { pac::SPI6: Spi6 }
+
+pub trait SpiExt: Sized + Instance {
+    fn spi<PINS: Pins<Self>>(
+        self,
+        pins: PINS,
+        mode: impl Into<Mode>,
+        freq: Hertz,
+        clocks: &Clocks,
+    ) -> Spi<Self, PINS, TransferModeNormal>;
+    fn spi_bidi<PINS: Pins<Self>>(
+        self,
+        pins: PINS,
+        mode: impl Into<Mode>,
+        freq: Hertz,
+        clocks: &Clocks,
+    ) -> Spi<Self, PINS, TransferModeBidi>;
+}
+
+impl<SPI: Instance> SpiExt for SPI {
+    fn spi<PINS: Pins<Self>>(
+        self,
+        pins: PINS,
+        mode: impl Into<Mode>,
+        freq: Hertz,
+        clocks: &Clocks,
+    ) -> Spi<Self, PINS, TransferModeNormal> {
+        Spi::new(self, pins, mode, freq, clocks)
+    }
+    fn spi_bidi<PINS: Pins<Self>>(
+        self,
+        pins: PINS,
+        mode: impl Into<Mode>,
+        freq: Hertz,
+        clocks: &Clocks,
+    ) -> Spi<Self, PINS, TransferModeBidi> {
+        Spi::new_bidi(self, pins, mode, freq, clocks)
+    }
+}
 
 impl<SPI, PINS> Spi<SPI, PINS, TransferModeNormal>
 where
@@ -169,7 +202,7 @@ where
         spi: SPI,
         mut pins: PINS,
         mode: impl Into<Mode>,
-        freq: impl Into<Hertz>,
+        freq: Hertz,
         clocks: &Clocks,
     ) -> Self {
         unsafe {
@@ -182,7 +215,7 @@ where
         pins.set_alt_mode();
 
         Self::_new(spi, pins)
-            .pre_init(mode.into(), freq.into(), SPI::clock(clocks))
+            .pre_init(mode.into(), freq, SPI::clock(clocks))
             .init()
     }
 
@@ -202,7 +235,7 @@ where
         spi: SPI,
         mut pins: PINS,
         mode: impl Into<Mode>,
-        freq: impl Into<Hertz>,
+        freq: Hertz,
         clocks: &Clocks,
     ) -> Self {
         unsafe {
@@ -215,7 +248,7 @@ where
         pins.set_alt_mode();
 
         Self::_new(spi, pins)
-            .pre_init(mode.into(), freq.into(), SPI::clock(clocks))
+            .pre_init(mode.into(), freq, SPI::clock(clocks))
             .init()
     }
 
@@ -308,7 +341,7 @@ where
         // disable SS output
         self.spi.cr2.write(|w| w.ssoe().clear_bit());
 
-        let br = match clock.0 / freq.0 {
+        let br = match clock.raw() / freq.raw() {
             0 => unreachable!(),
             1..=2 => 0b000,
             3..=5 => 0b001,
